@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-search.c,v 1.5 2006/02/04 16:36:51 pborelli Exp $
+ * $Id: gedit-commands-search.c,v 1.7 2006/02/25 17:48:38 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -709,6 +709,20 @@ gedit_cmd_search_find_prev (GtkAction   *action,
 	do_find_again (window, TRUE);
 }
 
+void
+gedit_cmd_search_clear_highlight (GtkAction   *action,
+				  GeditWindow *window)
+{
+	GeditDocument *doc;
+
+	gedit_debug (DEBUG_COMMANDS);
+
+	doc = gedit_window_get_active_document (window);
+	gedit_document_set_search_text (GEDIT_DOCUMENT (doc),
+					"",
+					GEDIT_SEARCH_DONT_SET_FLAGS);
+}
+
 void
 gedit_cmd_search_goto_line (GtkAction   *action,
 			    GeditWindow *window)
