@@ -33,7 +33,6 @@
 #include <glib/gi18n.h>
 
 #include "gedit-convert.h"
-#include "gedit-encodings.h"
 #include "gedit-debug.h"
 #include "gedit-prefs-manager.h"
 
@@ -51,31 +50,63 @@ static gchar *
 gedit_convert_to_utf8_from_charset (const gchar  *content,
 				    gsize         len,
 				    const gchar  *charset,
+				    gsize        *new_len,
 				    GError 	**error)
 {
 	gchar *utf8_content = NULL;
 	GError *conv_error = NULL;
-	gchar* converted_contents;
-	gsize bytes_written;
-
+	gchar* converted_contents = NULL;
+	gsize bytes_read;
+	
 	g_return_val_if_fail (content != NULL, NULL);
 	g_return_val_if_fail (len > 0, NULL);
 	g_return_val_if_fail (charset != NULL, NULL);
 
-	gedit_debug (DEBUG_UTILS, "Trying to convert from %s to UTF-8", charset);
-			
+	gedit_debug_message (DEBUG_UTILS, "Trying to convert from %s to UTF-8", charset);
+	
+	if (strcmp (charset, "UTF-8") == 0)
+	{
+		if (g_utf8_validate (content, len, NULL))
+		{
+			if (new_len != NULL)
+				*new_len = len;
+					
+			return g_strndup (content, len);
+		}
+		else	
+			g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE,
+				     "The file you are trying to open contains an invalid byte sequence.");
+		
+			return NULL;	
+	}
+	
 	converted_contents = g_convert (content, 
 					len, 
 					"UTF-8",
 					charset, 
-					NULL, 
-					&bytes_written,
+					&bytes_read, 
+					new_len,
 					&conv_error); 
-						
-	if ((conv_error != NULL) || !g_utf8_validate (converted_contents, bytes_written, NULL))
+	
+	gedit_debug_message (DEBUG_UTILS, "Bytes read: %d", bytes_read);
+		
+	/* There is no way we can avoid to run 	g_utf8_validate on the converted text.
+	 *
+	 * <paolo> hmmm... but in that case g_convert should fail 
+	 * <owen> paolo: g_convert() doesn't necessarily have the same definition
+         * <owen> GLib just uses the system's iconv
+         * <owen> paolo: I think we've explained what's going on. 
+         * I have to define it as NOTABUG since g_convert() isn't going to 
+         * start post-processing or checking what iconv() does and 
+         * changing g_utf8_valdidate() wouldn't be API compatible even if I 
+         * thought it was right
+	 */			
+	if ((conv_error != NULL) || 
+	    !g_utf8_validate (converted_contents, *new_len, NULL) ||
+	    (bytes_read != len))
 	{
-		gedit_debug (DEBUG_UTILS, "Couldn't convert from %s to UTF-8.",
-			     charset);
+		gedit_debug_message (DEBUG_UTILS, "Couldn't convert from %s to UTF-8.",
+			   	     charset);
 				
 		if (converted_contents != NULL)
 			g_free (converted_contents);
@@ -84,19 +115,20 @@ gedit_convert_to_utf8_from_charset (const gchar  *content,
 			g_propagate_error (error, conv_error);		
 		else
 		{
-			gedit_debug (DEBUG_UTILS, "The file you are trying to open contains "
-					"an invalid byte sequence.");
+			gedit_debug_message (DEBUG_UTILS, "The file you are trying to open contains "
+					     "an invalid byte sequence.");
 			
-			g_set_error (error, GEDIT_CONVERT_ERROR, GEDIT_CONVERT_ERROR_ILLEGAL_SEQUENCE,
-				     _("The file you are trying to open contains an invalid byte sequence."));
-		}
+			g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE,
+				     "The file you are trying to open contains an invalid byte sequence.");
+		}	
 	} 
 	else 
 	{
-		g_return_val_if_fail (converted_contents != NULL, NULL);
-
-		gedit_debug (DEBUG_UTILS, "Converted from %s to UTF-8.", charset);
+		gedit_debug_message (DEBUG_UTILS, "Converted from %s to UTF-8 (newlen = %d).", 
+				     charset, *new_len);
 	
+		g_return_val_if_fail (converted_contents != NULL, NULL); 
+
 		utf8_content = converted_contents;
 	}
 
@@ -104,16 +136,17 @@ gedit_convert_to_utf8_from_charset (const gchar  *content,
 }
 
 gchar *
-gedit_convert_to_utf8   (const gchar          *content, 
-			 gsize                 len,
-			 const GeditEncoding **encoding,
-			 GError              **error)
+gedit_convert_to_utf8 (const gchar          *content,
+		       gsize                 len,
+		       const GeditEncoding **encoding,
+		       gsize                *new_len,
+		       GError              **error)
 {
-	gedit_debug (DEBUG_UTILS, "");
-	
+	gedit_debug (DEBUG_UTILS);
+
 	g_return_val_if_fail (content != NULL, NULL);
 	g_return_val_if_fail (encoding != NULL, NULL);
-	
+
 	if (len < 0)
 		len = strlen (content);
 
@@ -128,46 +161,52 @@ gedit_convert_to_utf8   (const gchar          *content,
 		return gedit_convert_to_utf8_from_charset (content, 
 							   len,
 							   charset,
+							   new_len,
 							   error);
 	}
 	else
 	{
 		/* Automatically detect the encoding used */
 
-		GSList *encodings = NULL;
+		GSList *encodings;
 		GSList *start;
+		gchar *ret = NULL;
 
-		gedit_debug (DEBUG_UTILS, "Automatically detect the encoding used");
+		gedit_debug_message (DEBUG_UTILS,
+				     "Automatically detect the encoding used");
 
 		encodings = gedit_prefs_manager_get_auto_detected_encodings ();
 
 		if (encodings == NULL)
 		{
-			gedit_debug (DEBUG_UTILS, "encodings == NULL");
-			
+			gedit_debug_message (DEBUG_UTILS, "encodings == NULL");
+
 			if (g_utf8_validate (content, len, NULL))
 			{
-				gedit_debug (DEBUG_UTILS, "validate OK.");
+				gedit_debug_message (DEBUG_UTILS, "validate OK.");
 
+				if (new_len != NULL)
+					*new_len = len;
+				
 				return g_strndup (content, len);
 			}
 			else
 			{
-				gedit_debug (DEBUG_UTILS, "validate failed.");
+				gedit_debug_message (DEBUG_UTILS, "validate failed.");
 
 				g_set_error (error, GEDIT_CONVERT_ERROR, 
 					     GEDIT_CONVERT_ERROR_AUTO_DETECTION_FAILED,
-				 	     _("gedit was not able to automatically determine "
-					       "the encoding of the file you want to open."));
+				 	     "gedit was not able to automatically determine "
+					     "the encoding of the file you want to open.");
 
 				return NULL;
 			}
 		}
 
-		gedit_debug (DEBUG_UTILS, "encodings != NULL");
+		gedit_debug_message (DEBUG_UTILS, "encodings != NULL");
 
 		start = encodings;
-		
+
 		while (encodings != NULL) 
 		{
 			const GeditEncoding *enc;
@@ -176,38 +215,48 @@ gedit_convert_to_utf8   (const gchar          *content,
 
 			enc = (const GeditEncoding *)encodings->data;
 
-			gedit_debug (DEBUG_UTILS, "Get charset");
+			gedit_debug_message (DEBUG_UTILS, "Get charset");
 
 			charset = gedit_encoding_get_charset (enc);
 			g_return_val_if_fail (charset != NULL, NULL);
 
-			gedit_debug (DEBUG_UTILS, "Trying to convert %ld bytes of data from ' %s'to 'UTF-8'.", 
-					len, charset);
+			gedit_debug_message (DEBUG_UTILS, "Trying to convert %lu bytes of data from '%s' to 'UTF-8'.", 
+					(unsigned long) len, charset);
 
-			utf8_content = gedit_convert_to_utf8_from_charset (content, len, charset, NULL);
+			utf8_content = gedit_convert_to_utf8_from_charset (content, 
+									   len, 
+									   charset, 
+									   new_len,
+									   NULL);
 
 			if (utf8_content != NULL) 
 			{
 				*encoding = enc;
+				ret = utf8_content;
 
-				return utf8_content;
+				break;
 			}
 
 			encodings = g_slist_next (encodings);
 		}
 
-		gedit_debug (DEBUG_UTILS, "gedit has not been able to automatically determine the encoding of "
-			     "the file you want to open.");
-		
-		g_set_error (error, GEDIT_CONVERT_ERROR,
-			     GEDIT_CONVERT_ERROR_AUTO_DETECTION_FAILED,
-		 	     _("gedit was not able to automatically determine "
-			       "the encoding of the file you want to open."));
+		if (ret == NULL)
+		{
+			gedit_debug_message (DEBUG_UTILS, "gedit has not been able to automatically determine the encoding of "
+					     "the file you want to open.");
+
+			g_set_error (error, GEDIT_CONVERT_ERROR,
+				     GEDIT_CONVERT_ERROR_AUTO_DETECTION_FAILED,
+			 	     "gedit was not able to automatically determine "
+				     "the encoding of the file you want to open.");
+		}
 
 		g_slist_free (start);
+
+		return ret;
 	}
-	
-	return NULL;
+
+	g_return_val_if_reached (NULL);
 }
 
 gchar *
@@ -221,7 +270,7 @@ gedit_convert_from_utf8 (const gchar          *content,
 	gchar  *converted_contents = NULL;
 	gsize   bytes_written = 0;
 
-	gedit_debug (DEBUG_UTILS, "");
+	gedit_debug (DEBUG_UTILS);
 	
 	g_return_val_if_fail (content != NULL, NULL);
 	g_return_val_if_fail (g_utf8_validate (content, len, NULL), NULL);
@@ -243,8 +292,8 @@ gedit_convert_from_utf8 (const gchar          *content,
 
 	if (conv_error != NULL)
 	{
-		gedit_debug (DEBUG_UTILS, "Cannot convert from UTF-8 to %s",
-				gedit_encoding_get_charset (encoding));
+		gedit_debug_message (DEBUG_UTILS, "Cannot convert from UTF-8 to %s",
+				     gedit_encoding_get_charset (encoding));
 
 		if (converted_contents != NULL)
 		{
