@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document-saver.c,v 1.4 2006/01/30 14:14:22 pborelli Exp $
+ * $Id: gedit-document-saver.c,v 1.5 2006/02/01 08:32:36 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -833,6 +833,7 @@ static void
 save_local_file (GeditDocumentSaver *saver)
 {
 	GSourceFunc next_phase;
+	GnomeVFSResult result;
 
 	/* saving start */
 	g_signal_emit (saver,
@@ -863,7 +864,7 @@ save_local_file (GeditDocumentSaver *saver)
 	}
 
 	/* else error */
-	GnomeVFSResult result = gnome_vfs_result_from_errno (); //may it happen that no errno?
+	result = gnome_vfs_result_from_errno (); //may it happen that no errno?
 
 	g_set_error (&saver->priv->error,
 		     GEDIT_DOCUMENT_ERROR,
